use circular_buffer::CircularBuffer;
use std::mem::MaybeUninit;

#[test]
fn test_basic() {
    let mut storage = [MaybeUninit::<i32>::uninit(); 5];
    let buffer = CircularBuffer::from_storage(&mut storage);

    buffer.push_all([1, 2, 3]).unwrap();
    assert_eq!(vec![1, 2, 3], buffer.execute(|r| r.to_vec()));
    buffer.push_all([1, 2, 3, 4]).unwrap();
    assert_eq!(vec![1, 2], buffer.execute(|r| r.to_vec()));
    assert_eq!(vec![3, 4], buffer.execute(|r| r.to_vec()));
    assert!(buffer.execute(|r| r.is_empty()));
    buffer.push_all(0..4).unwrap();
    assert_eq!(Some(0), buffer.pop());
    assert_eq!(vec![1, 2, 3], buffer.collect::<Vec<_>>());
    assert_eq!(None, (&buffer).next());
    buffer.push_all([1, 2, 3]).unwrap();
    assert_eq!(Some(1), (&buffer).next());
    assert_eq!(vec![2, 3], buffer.collect::<Vec<_>>());
}

#[test]
fn test_str() {
    use std::fmt::Write;
    let mut storage = [MaybeUninit::<u8>::uninit(); 100];
    let buffer = CircularBuffer::from_storage(&mut storage);
    write!(&buffer, "40 + 2 = {}", 40 + 2).unwrap();
    let content = String::from_utf8(buffer.collect()).unwrap();
    assert_eq!("40 + 2 = 42", content);
}
