use circular_buffer::CircularBuffer;
use std::fmt::Write;
use std::mem::MaybeUninit;
use std::sync::Arc;
use std::thread;
use std::time::Duration;

#[test]
fn test_concurrency() {
    static mut STORAGE: [MaybeUninit<u8>; 100] = [MaybeUninit::uninit(); 100];
    let buffer = Arc::new(CircularBuffer::from_storage(unsafe { &mut STORAGE }));
    let writer = thread::spawn({
        let buffer = Arc::clone(&buffer);
        move || {
            let mut buffer = buffer.as_ref();
            for _ in 0..10_000 {
                while buffer.write_available() < 7 {
                    thread::sleep(Duration::from_micros(1));
                }
                write!(&mut buffer, "foobar!").unwrap();
            }
        }
    });

    for _ in 0..10_000 {
        let mut result = String::new();
        while result.len() != 7 {
            while buffer.is_empty() {
                thread::sleep(Duration::from_micros(1));
            }
            buffer.execute_max(7 - result.len(), |s| {
                result.push_str(std::str::from_utf8(s).unwrap())
            })
        }
        assert_eq!("foobar!", &result);
    }
    writer.join().unwrap()
}
